@@ -170,7 +170,7 @@ totemPlugin::~totemPlugin ()
 		dbus_g_proxy_disconnect_signal (mBusProxy,
 						"NameOwnerChanged",
 						G_CALLBACK (NameOwnerChangedCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 		g_object_unref (mBusProxy);
 		mBusProxy = NULL;
 	}
@@ -349,7 +349,7 @@ totemPlugin::ViewerFork ()
 	/* Don't wait forever! */
 	const PRUint32 kViewerTimeout = 30 * 1000; /* ms */
 	nsresult rv = mTimer->InitWithFuncCallback (ViewerForkTimeoutCallback,
-						    NS_REINTERPRET_CAST (void*, this),
+						    reinterpret_cast<void*>(this),
 						    kViewerTimeout,
 						    nsITimer::TYPE_ONE_SHOT);
 	if (NS_FAILED (rv)) {
@@ -424,7 +424,7 @@ totemPlugin::ViewerSetup ()
 	dbus_g_proxy_connect_signal (mViewerProxy,
 				     "ButtonPress",
 				     G_CALLBACK (ButtonPressCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	dbus_g_proxy_add_signal (mViewerProxy,
@@ -433,7 +433,7 @@ totemPlugin::ViewerSetup ()
 	dbus_g_proxy_connect_signal (mViewerProxy,
 				     "StopStream",
 				     G_CALLBACK (StopStreamCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	if (mHidden) {
@@ -472,11 +472,11 @@ totemPlugin::ViewerCleanup ()
 		dbus_g_proxy_disconnect_signal (mViewerProxy,
 						"ButtonPress",
 						G_CALLBACK (ButtonPressCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 		dbus_g_proxy_disconnect_signal (mViewerProxy,
 						"StopStream",
 						G_CALLBACK (StopStreamCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 
 		g_object_unref (mViewerProxy);
 		mViewerProxy = NULL;
@@ -519,7 +519,7 @@ totemPlugin::ViewerSetWindow ()
 		dbus_g_proxy_begin_call (mViewerProxy,
 					 "SetWindow",
 					 ViewerSetWindowCallback,
-					 NS_REINTERPRET_CAST (void*, this),
+					 reinterpret_cast<void*>(this),
 					 NULL,
 #if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
 					 G_TYPE_STRING, mControls.get (),
@@ -767,7 +767,7 @@ totemPlugin::RequestStream (PRBool aForceViewer)
 			dbus_g_proxy_begin_call (mViewerProxy,
 						 "OpenStream",
 						 ViewerOpenStreamCallback,
-						 NS_REINTERPRET_CAST (void*, this),
+						 reinterpret_cast<void*>(this),
 						 NULL,
 						 G_TYPE_STRING, spec.get (),
 						 G_TYPE_STRING, baseSpec.get (),
@@ -777,7 +777,7 @@ totemPlugin::RequestStream (PRBool aForceViewer)
 			dbus_g_proxy_begin_call (mViewerProxy,
 						 "OpenURI",
 						 ViewerOpenURICallback,
-						 NS_REINTERPRET_CAST (void*, this),
+						 reinterpret_cast<void*>(this),
 						 NULL,
 						 G_TYPE_STRING, spec.get (),
 						 G_TYPE_STRING, baseSpec.get (),
@@ -785,6 +785,12 @@ totemPlugin::RequestStream (PRBool aForceViewer)
 	}
 
 	/* FIXME: start playing in the callbacks ! */
+
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (mScriptable) {
+		mScriptable->mPluginState = totemScriptablePlugin::eState_Playable;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
 
 void
@@ -805,6 +811,12 @@ totemPlugin::UnsetStream ()
 	/* Calling DestroyStream should already have set this to NULL */
 	NS_ASSERTION (!mStream, "Should not have a stream anymore");
 	mStream = nsnull; /* just to make sure */
+
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (mScriptable) {
+		mScriptable->mPluginState = totemScriptablePlugin::eState_Waiting;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
 
 /* Callbacks */
@@ -816,7 +828,7 @@ totemPlugin::NameOwnerChangedCallback (DBusGProxy *proxy,
 				       const char *aNewOwner,
 				       void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	plugin->NameOwnerChanged (aName, aOldOwner, aNewOwner);
 }
@@ -825,7 +837,7 @@ totemPlugin::NameOwnerChangedCallback (DBusGProxy *proxy,
 totemPlugin::ViewerForkTimeoutCallback (nsITimer *aTimer,
 				        void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("ViewerForkTimeoutCallback");
 
@@ -842,7 +854,7 @@ totemPlugin::ButtonPressCallback (DBusGProxy *proxy,
 				  guint aButton,
 			          void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("ButtonPress signal received");
 
@@ -853,7 +865,7 @@ totemPlugin::ButtonPressCallback (DBusGProxy *proxy,
 totemPlugin::StopStreamCallback (DBusGProxy *proxy,
 			         void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("StopStream signal received");
 
@@ -865,7 +877,7 @@ totemPlugin::ViewerSetWindowCallback (DBusGProxy *aProxy,
 				      DBusGProxyCall *aCall,
 				      void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("SetWindow reply");
 
@@ -889,7 +901,7 @@ totemPlugin::ViewerOpenStreamCallback (DBusGProxy *aProxy,
 				       DBusGProxyCall *aCall,
 				       void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("OpenStream reply");
 
@@ -928,7 +940,14 @@ totemPlugin::ViewerOpenStreamCallback (DBusGProxy *aProxy,
 		plugin->mExpectingStream = PR_FALSE;
 
 		D ("GetURLNotify '%s' failed with error %d", spec.get (), err);
+		return;
 	}
+
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (plugin->mScriptable) {
+		plugin->mScriptable->mPluginState = totemScriptablePlugin::eState_Playable;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
 
 /* static */ void PR_CALLBACK
@@ -936,7 +955,7 @@ totemPlugin::ViewerOpenURICallback (DBusGProxy *aProxy,
 				    DBusGProxyCall *aCall,
 				    void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("OpenURI reply");
 
@@ -951,6 +970,12 @@ totemPlugin::ViewerOpenURICallback (DBusGProxy *aProxy,
 		return;
 	}
 
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (plugin->mScriptable) {
+		plugin->mScriptable->mPluginState = totemScriptablePlugin::eState_Playable;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
+
 	/* FIXME this isn't the best way... */
 	if (plugin->mAutostart) {
 		plugin->DoCommand (TOTEM_COMMAND_PLAY);
@@ -1460,8 +1485,8 @@ totemPlugin::Init (NPMIMEType mimetype,
 	NPError err;
 	err = CallNPN_GetValueProc (sNPN.getvalue,
 				    mInstance, NPNVserviceManager,
-				    NS_REINTERPRET_CAST (void *,
-							 NS_REINTERPRET_CAST (void **, &mServiceManager)));
+				    reinterpret_cast<void *>
+                        (reinterpret_cast<void **>(&mServiceManager)));
 	if (err != NPERR_NO_ERROR || !mServiceManager) {
 		D ("Failed to get the service manager");
 		return NPERR_GENERIC_ERROR;
@@ -1470,7 +1495,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 	nsresult rv;
 	rv = mServiceManager->GetServiceByContractID (NS_IOSERVICE_CONTRACTID,
 						      NS_GET_IID (nsIIOService),
-						      NS_REINTERPRET_CAST (void **, &mIOService));
+						      reinterpret_cast<void **>(&mIOService));
 	if (NS_FAILED (rv) || !mIOService) {
 		D ("Failed to get IO service");
 		return NPERR_GENERIC_ERROR;
@@ -1478,8 +1503,8 @@ totemPlugin::Init (NPMIMEType mimetype,
 
 	err = CallNPN_GetValueProc (sNPN.getvalue,
 				    mInstance, NPNVDOMElement,
-				    NS_REINTERPRET_CAST (void *,
-						         NS_REINTERPRET_CAST (void **, &mPluginDOMElement)));
+				    reinterpret_cast<void *>
+                        (reinterpret_cast<void **>(&mPluginDOMElement)));
 	if (err != NPERR_NO_ERROR || !mPluginDOMElement) {
 		D ("Failed to get our DOM Element");
 		return NPERR_GENERIC_ERROR;
@@ -1537,7 +1562,7 @@ totemPlugin::Init (NPMIMEType mimetype,
         rv = compMan->CreateInstanceByContractID (NS_TIMER_CONTRACTID,
                                                   nsnull,
                                                   NS_GET_IID (nsITimer),
-                                                  NS_REINTERPRET_CAST (void **, &mTimer));
+                                                  reinterpret_cast<void **>(&mTimer));
         if (NS_FAILED (rv) || !mTimer) {
                 D ("Failed to create timer: rv=%x", rv);
                 return NPERR_GENERIC_ERROR;
@@ -1569,7 +1594,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 	dbus_g_proxy_connect_signal (mBusProxy,
 				     "NameOwnerChanged",
 				     G_CALLBACK (NameOwnerChangedCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	/* Find the "real" mime-type */
@@ -1614,17 +1639,6 @@ totemPlugin::Init (NPMIMEType mimetype,
 	mHidden = g_hash_table_lookup (args, "hidden") != NULL &&
 		  GetBooleanValue (args, "hidden", PR_TRUE);
 
-#ifdef TOTEM_GMP_PLUGIN
-	if (height == 40) {
-		mAudioOnly = PR_TRUE;
-	}
-#endif /* TOTEM_GMP_PLUGIN */
-#if defined(TOTEM_NARROWSPACE_PLUGIN) || defined (TOTEM_BASIC_PLUGIN)
-	if (height <= 16) {
-		mAudioOnly = PR_TRUE;
-	}
-#endif /* TOTEM_NARROWSPACE_PLUGIN */
-
 	/* Most for RealAudio streams, but also used as a replacement for
 	 * HIDDEN=TRUE attribute.
 	 */
@@ -1783,6 +1797,19 @@ totemPlugin::Init (NPMIMEType mimetype,
 	//FIXME handle starttime and endtime
 	// http://www.htmlcodetutorial.com/embeddedobjects/_EMBED_STARTTIME.html
 
+	/* Minimum heights of the different plugins, note that the
+	 * controllers need to be showing, otherwise it's useless */
+#ifdef TOTEM_GMP_PLUGIN
+	if (height == 40 && !mControllerHidden) {
+		mAudioOnly = PR_TRUE;
+	}
+#endif /* TOTEM_GMP_PLUGIN */
+#if defined(TOTEM_NARROWSPACE_PLUGIN) || defined (TOTEM_BASIC_PLUGIN)
+	if (height <= 16 && !mControllerHidden) {
+		mAudioOnly = PR_TRUE;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
+
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 	/* We need to autostart if we're using an HREF
 	 * otherwise the start image isn't shown */
@@ -1916,6 +1943,12 @@ totemPlugin::NewStream (NPMIMEType type,
 		mStreamType = NP_ASFILE;
 	}
 
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (mScriptable) {
+		mScriptable->mPluginState = totemScriptablePlugin::eState_Loading;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
+
 	mStream = stream;
 
 	mCheckedForPlaylist = PR_FALSE;
@@ -2052,9 +2085,6 @@ totemPlugin::StreamAsFile (NPStream *stream,
 
 	D ("StreamAsFile filename '%s'", fname);
 
-	/* FIXME: this reads the whole file into memory... try to
-	 * find a way to avoid it.
-	 */
 	if (!mCheckedForPlaylist) {
 		mIsPlaylist = totem_pl_parser_can_parse_from_filename
 				(fname, TRUE) != FALSE;
@@ -2106,14 +2136,30 @@ totemPlugin::StreamAsFile (NPStream *stream,
 					    G_TYPE_STRING, baseSpec.get (),
 					    G_TYPE_INVALID,
 					    G_TYPE_INVALID);
-	} else {
+	}
+	/* If the file has finished streaming from the network
+	 * and is on the disk, then we should be able to play
+	 * it back from the cache, rather than just stopping there */
+	else {
 		D ("mBytesStreamed %u", mBytesStreamed);
+		retval = dbus_g_proxy_call (mViewerProxy,
+					    "SetLocalCache",
+					    &error,
+					    G_TYPE_STRING, fname,
+					    G_TYPE_INVALID,
+					    G_TYPE_INVALID);
 	}
 
 	if (!retval) {
 		g_warning ("Viewer error: %s", error->message);
 		g_error_free (error);
+		return;
 	}
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+	if (mScriptable) {
+		mScriptable->mPluginState = totemScriptablePlugin::eState_Complete;
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
     
 void
@@ -2158,11 +2204,15 @@ totemPlugin::GetScriptable (void *_retval)
 		if (!mScriptable)
 			return NPERR_OUT_OF_MEMORY_ERROR;
 
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+		mScriptable->mPluginState = totemScriptablePlugin::eState_Waiting;
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
+
 		NS_ADDREF (mScriptable);
 	}
 
 	nsresult rv = mScriptable->QueryInterface (NS_GET_IID (nsISupports),
-						   NS_REINTERPRET_CAST (void **, _retval));
+						   reinterpret_cast<void **>(_retval));
 
 	return NS_SUCCEEDED (rv) ? NPERR_NO_ERROR : NPERR_GENERIC_ERROR;
 }
