@@ -28,7 +28,7 @@
 #ifndef HAVE_BACON_VIDEO_WIDGET_H
 #define HAVE_BACON_VIDEO_WIDGET_H
 
-#include <gtk/gtkbox.h>
+#include <gtk/gtkeventbox.h>
 
 /* for optical disc enumeration type */
 #include "totem-disc.h"
@@ -46,13 +46,13 @@ typedef struct BaconVideoWidgetPrivate BaconVideoWidgetPrivate;
 typedef struct BaconVideoWidgetCommon BaconVideoWidgetCommon;
 
 typedef struct {
-	GtkBox parent;
+	GtkEventBox parent;
 	BaconVideoWidgetCommon *com;
 	BaconVideoWidgetPrivate *priv;
 } BaconVideoWidget;
 
 typedef struct {
-	GtkBoxClass parent_class;
+	GtkEventBoxClass parent_class;
 
 	void (*error) (GtkWidget *bvw, const char *message,
                        gboolean playback_stopped, gboolean fatal);
@@ -145,8 +145,8 @@ void bacon_video_widget_close                    (BaconVideoWidget *bvw);
 /* Audio volume */
 gboolean bacon_video_widget_can_set_volume       (BaconVideoWidget *bvw);
 void bacon_video_widget_set_volume               (BaconVideoWidget *bvw,
-						  int volume);
-int bacon_video_widget_get_volume                (BaconVideoWidget *bvw);
+						  double volume);
+double bacon_video_widget_get_volume             (BaconVideoWidget *bvw);
 
 /* Properties */
 void bacon_video_widget_set_logo		 (BaconVideoWidget *bvw,
@@ -173,12 +173,12 @@ void bacon_video_widget_set_connection_speed     (BaconVideoWidget *bvw,
 int bacon_video_widget_get_connection_speed      (BaconVideoWidget *bvw);
 
 gboolean bacon_video_widget_can_play             (BaconVideoWidget *bvw,
-						  MediaType type);
+						  TotemDiscMediaType type);
 
 #define MEDIA_TYPE_DVB MEDIA_TYPE_NUM_TYPES
 
 gchar **bacon_video_widget_get_mrls		 (BaconVideoWidget *bvw,
-						  MediaType type,
+						  TotemDiscMediaType type,
 						  const char *device);
 void bacon_video_widget_set_subtitle_font	 (BaconVideoWidget *bvw,
 						  const char *font);
@@ -190,6 +190,7 @@ typedef enum {
 	BVW_INFO_TITLE,
 	BVW_INFO_ARTIST,
 	BVW_INFO_YEAR,
+	BVW_INFO_COMMENT,
 	BVW_INFO_ALBUM,
 	BVW_INFO_DURATION,
 	BVW_INFO_TRACK_NUMBER,
